"""
Daves utilities is a collection of useful python functions which I would like to
provide to anybody who is interested in using them.
"""

__version__ = "0.0.14"

from daves_utilities.for_long import for_long
from daves_utilities.fun_save import fun_save
from daves_utilities.is_equal import is_equal
from daves_utilities.print_structure import print_str

__all__ = ["for_long", "fun_save", "is_equal", "print_str"]
